from abc import ABC, abstractmethod
from ..messages import actions, game_updates, message_protocol
from . import PlayerId
from .. import Value

class PlayerActions(ABC): 
    @abstractmethod
    def _draw_or_trade(self) -> actions.PlayerTurnDecision: ...

    @abstractmethod
    def _trade(self) -> actions.InitialTrade: ...
    
    @abstractmethod
    def _provide_bidding(self, state: game_updates.AuctionRound) -> actions.Bidding: ...
    
    @abstractmethod
    def _buy_or_sell(self, state: game_updates.AuctionRound) -> actions.AuctionDecision: ...

    @abstractmethod
    def _send_money_to_player(self, player: PlayerId, amount: Value) -> actions.SendMoney: ...
    
    @abstractmethod
    def _respond_to_trade(self, offer: game_updates.TradeOffer) -> actions.TradeOpponentDecision: ...
    
    @abstractmethod
    def _receive_game_update(self, update: game_updates.GameUpdate) -> actions.NoAction: ...



from typing import Union
from .actions import (
    AuctionDecision,
    Bidding,
    InitialTrade,
    NoAction,
    PlayerTurnDecision,
    SendMoney,
    TradeOpponentDecision,
)
from .game_updates import AuctionRound, GameUpdate, TradeOffer
from ..player import PlayerId
from .. import Value

class ActionMessage:
    class PlayerTurnDecision(ActionMessage):
        decision: PlayerTurnDecision
        def __init__(self, decision: PlayerTurnDecision) -> None: ...
        __match_args__ = ("decision",)

    class InitialTrade(ActionMessage):
        decision: InitialTrade
        def __init__(self, decision: InitialTrade) -> None: ...
        __match_args__ = ("decision",)

    class Bidding(ActionMessage):
        decision: Bidding
        def __init__(self, decision: Bidding) -> None: ...
        __match_args__ = ("decision",)

    class AuctionDecision(ActionMessage):
        decision: AuctionDecision
        def __init__(self, decision: AuctionDecision) -> None: ...
        __match_args__ = ("decision",)

    class SendMoney(ActionMessage):
        decision: SendMoney
        def __init__(self, decision: SendMoney) -> None: ...
        __match_args__ = ("decision",)

    class TradeOpponentDecision(ActionMessage):
        decision: TradeOpponentDecision
        def __init__(self, decision: TradeOpponentDecision) -> None: ...
        __match_args__ = ("decision",)

    class NoAction(ActionMessage):
        decision: NoAction
        def __init__(self, decision: NoAction) -> None: ...
        __match_args__ = ("decision",)

class StateMessage:
    class DrawOrTrade(StateMessage):
        def __init__(self) -> None: ...

    class Trade(StateMessage):
        def __init__(self) -> None: ...

    class ProvideBidding(StateMessage):
        state: AuctionRound
        def __init__(self, state: AuctionRound) -> None: ...
        __match_args__ = ("state",)

    class BuyOrSell(StateMessage):
        state: AuctionRound
        def __init__(self, state: AuctionRound) -> None: ...
        __match_args__ = ("state",)

    class SendMoney(StateMessage):
        player_id: PlayerId
        amount: Value
        def __init__(self, player_id: PlayerId, amount: Value) -> None: ...
        __match_args__ = ("player_id", "amount")

    class RespondToTrade(StateMessage):
        offer: TradeOffer
        def __init__(self, offer: TradeOffer) -> None: ...
        __match_args__ = ("offer",)

    class GameUpdate(StateMessage):
        update: GameUpdate
        def __init__(self, update: GameUpdate) -> None: ...
        __match_args__ = ("update",)



from enum import Enum
from ..player import PlayerId
from ..player.wallet import Wallet
from ..animals import Animal, AnimalSet
from .. import Money, Value, Points
from .actions import Bidding

class AuctionRound:
    host: PlayerId
    animal: Animal
    bids: list[tuple[PlayerId, Bidding]]

    def __init__(
        self,
        host: PlayerId,
        animal: Animal,
        bids: list[tuple[PlayerId, Bidding]],
    ) -> None: ...

class TradeOffer:
    challenger: PlayerId
    animal: Animal
    animal_count: int
    challenger_card_offer: int

    def __init__(
        self,
        challenger: PlayerId,
        animal: Animal,
        animal_count: int,
        challenger_card_offer: int,
    ) -> None: ...

class GameUpdate:
    class Auction(GameUpdate):
        auction_kind: AuctionKind
        def __init__(self, auction_kind: AuctionKind) -> None: ...
        __match_args__ = ("auction_kind",)

    class Trade(GameUpdate):
        challenger: PlayerId
        opponent: PlayerId
        animal: Animal
        animal_count: int
        receiver: PlayerId
        money_trade: MoneyTrade

        def __init__(
            self,
            challenger: PlayerId,
            opponent: PlayerId,
            animal: Animal,
            animal_count: int,
            receiver: PlayerId,
            money_trade: MoneyTrade,
        ) -> None: ...

        __match_args__ = (
            "challenger",
            "opponent",
            "animal",
            "animal_count",
            "receiver",
            "money_trade",
        )

    class Start(GameUpdate):
        wallet: Wallet
        players_in_turn_order: list[PlayerId]
        animals: list[AnimalSet]

        def __init__(
            self,
            wallet: Wallet,
            players_in_turn_order: list[PlayerId],
            animals: list[AnimalSet],
        ) -> None: ...

        __match_args__ = (
            "wallet",
            "players_in_turn_order",
            "animals",
        )

    class End(GameUpdate):
        ranking: list[tuple[PlayerId, Points]]
        def __init__(self, ranking: list[tuple[PlayerId, Points]]) -> None: ...

        __match_args__ = ("ranking",)

    class ExposePlayer(GameUpdate):
        player: PlayerId
        wallet: Wallet

        def __init__(self, player: PlayerId, wallet: Wallet) -> None: ...

        __match_args__ = (
            "player",
            "wallet",
        )

    class Inflation(GameUpdate):
        money: Money
        def __init__(self, money: Money) -> None: ...

        __match_args__ = ("money",)

class AuctionKind:
    class NoBiddings(AuctionKind):
        host_id: PlayerId
        animal: Animal
        def __init__(self, host_id: PlayerId, animal: Animal) -> None: ...

        __match_args__ = ("host_id", "animal")

    class NormalAuction(AuctionKind):
        rounds: AuctionRound
        from_player: PlayerId
        to_player: PlayerId
        money_transfer: MoneyTransfer

        def __init__(
            self,
            rounds: AuctionRound,
            from_player: PlayerId,
            to_player: PlayerId,
            money_transfer: MoneyTransfer,
        ) -> None: ...

        __match_args__ = ("rounds", "from_player", "to_player", "money_transfer")

class MoneyTransfer:
    class Public(MoneyTransfer):
        card_amount: int
        min_value: Value

        def __init__(
            self,
            card_amount: int,
            min_value: Value,
        ) -> None: ...

        __match_args__ = ("card_amount", "min_value")

class MoneyTrade:
    class Public(MoneyTrade):
        challenger_card_offer: int
        opponent_card_offer: tuple[int | None]

        def __init__(
            self,
            challenger_card_offer: int,
            opponent_card_offer: tuple[int | None],
        ) -> None: ...

        __match_args__ = ("card_amount", "min_value")

    class Private(MoneyTrade):
        challenger_card_offer: list[Money]
        opponent_card_offer: tuple[list[Money] | None]

        def __init__(
            self,
            challenger_card_offer: list[Money],
            opponent_card_offer: tuple[list[Money] | None],
        ) -> None: ...

        __match_args__ = ("challenger_card_offer", "opponent_card_offer")



from . import Value

class Animal:
    value: Value
    def __init__(self, value: Value) -> None: ...

class AnimalSet:
    animal: Animal
    inflation: list[Value]
    draw_count: int
    animals: list[Animal]

    def __init__(
        self,
        animal: Animal,
        inflation: list[Value],
        draw_count: int,
        animals: list[Animal],
    ) -> None: ...
